#! /usr/bin/perl
#

use strict;

my $InputFileName = shift;
my $OutputFileName = shift;

my $versionPrint = $0 . "\n";
$versionPrint .=  <<'END';
$Date: 2007-06-11 13:24:06 +0200 $
$Id: wordhtm2glossaryxml.pl 841 2007-06-11 11:24:06Z verhas $
$Author: verhas $
$Rev: 841 $
END

$versionPrint =~ s{\$}{}g;
print $versionPrint;

my $infoConstants = <<'END';
    <STUDENTCANPOST>1</STUDENTCANPOST>
    <ALLOWDUPLICATEDENTRIES>0</ALLOWDUPLICATEDENTRIES>
    <DISPLAYFORMAT>dictionary</DISPLAYFORMAT>
    <SHOWSPECIAL>1</SHOWSPECIAL>
    <SHOWALPHABET>1</SHOWALPHABET>
    <SHOWALL>1</SHOWALL>
    <ALLOWCOMMENTS>0</ALLOWCOMMENTS>
    <USEDYNALINK>1</USEDYNALINK>
    <DEFAULTAPPROVAL>1</DEFAULTAPPROVAL>
    <GLOBALGLOSSARY>0</GLOBALGLOSSARY>
    <ENTBYPAGE>10</ENTBYPAGE>
END

if( open(F,"<glossaryflags.xml") ){
  undef $/;
  $infoConstants = <F>;
  close F;
}

my  $entryConstants = <<'END';
        <FORMAT>1</FORMAT>
        <USEDYNALINK>0</USEDYNALINK>
        <CASESENSITIVE>0</CASESENSITIVE>
        <FULLMATCH>0</FULLMATCH>
        <TEACHERENTRY>1</TEACHERENTRY>
END

if( open(F,"<entryflags.xml") ){
  undef $/;
  $entryConstants = <F>;
  close F;
}

if( ! defined $InputFileName ){
	print "Hasznalat modja: $0 bemeno-file [kimeno-file]\n";
  print "\n";
	exit(1);
}
if( ! defined $OutputFileName ){
	$OutputFileName = $InputFileName;
	$OutputFileName =~ s/\.[^.]*$/.xml/;
}
print "Konvertalas: $InputFileName -> $OutputFileName\n";

use open IN => "::encoding(windows-1250)";
if( ! open(F,$InputFileName) ) {
	print "Nem lehet megnyitni olvasasra a $InputFileName fajlt\n";
	exit(1);
}

undef $/;
my $InputFile = <F>;
close F;

$InputFile =~ s/\&nbsp\;//g;

my $TransferBuffer = $InputFile;

# delete all tag attributes
$TransferBuffer =~ s/\<\s*(\w+)\s*[^>]*\>/<$1>/g;
# delete all comments
my $startIndex = index($TransferBuffer,'<!--');
while( $startIndex > -1  ){
	my $endIndex = index($TransferBuffer,'-->');
  if( $endIndex > -1 ){
	  $TransferBuffer = substr($TransferBuffer,0,$startIndex) . substr($TransferBuffer,$endIndex+3);
  	$startIndex = index($TransferBuffer,'<!--');
  }else{
    $startIndex = -1;
  }
}
#delete all unneeded tags
$TransferBuffer =~ s/\<span\>//gi;
$TransferBuffer =~ s/\<\/span\>//gi;
$TransferBuffer =~ s/\<o\>//gi;
$TransferBuffer =~ s/\<\/o\:?\w*\>//gi;
# delete all <p> and
$TransferBuffer =~ s{<p>}{}gi;
#    convert all </p> to <p> to make it from xhtml to html
$TransferBuffer =~ s{</p>}{<p>}gi;

$TransferBuffer =~ m{<title>(.*)</title>}i;
my $title = $1;

$startIndex = index($TransferBuffer,'<body>');
my $IntroPart = substr($TransferBuffer,$startIndex + length('<body>'));

$IntroPart =~ s{<b>%.*$}{}si;

$IntroPart = htmlescape($IntroPart);

# first we collect the terms, definitions and aliases into arrays
# and then the arrays are used to print out the XML format
my @term = (); # the terms
my @defs = (); # the definitions
my @alas = (); # aliases comma separated

# seek the first term
$startIndex = index($TransferBuffer,'%');
my $jndex = 0; # term, defs and alas array index
while( $startIndex > -1 ){
	
  #
  # The format of a term and definition:
  #
  #         % This is the term : this is the definition #
  #	

  # chop off any data that may be before the defintion
  $TransferBuffer = substr($TransferBuffer,$startIndex);
  # seek the end of the definition
  my $endIndex = index($TransferBuffer,'#');
  # $termAndDefinition will contain only the part that is between the % and the #
  my $termAndDefinition = substr($TransferBuffer,0, $endIndex - 1 );

  # the term starts after the %
  my $termStart = length('%');
  # the term ends with the : character
  my $termEnd   = index($termAndDefinition,':');
  # get the term string itself
  my $termAndAliases = substr($termAndDefinition,$termStart,$termEnd-$termStart);
  # aliases can be defined between ( and ) after the term before the :
  $termAndAliases =~ /\((.*)\)/;
  # if there are aliases defined
  if( defined $1 ){
  	# store the aliases
  	$alas[$jndex] = $1;
  	# chop off the aliases
  	$termAndAliases =~ s/\(.*//s;
  }else{
  	$alas[$jndex] = undef;
  }
  # termAndAliases now contains the terms only w/o the aliases
  $term[$jndex] = $termAndAliases;
  # delete the space from the start and from the end of the term description if any
  $term[$jndex] =~ s{^\s*}{}s;
  $term[$jndex] =~ s{\s*$}{}s;

  # first copy the term and the definition  
  my $definition = $termAndDefinition;
  # delete the term and let the definition remain
  $definition =~ s{^.*?\:}{}s;
  
  # delete any trailing and leading space
  $definition =~ s{\s*$}{}s;
  $definition =~ s{^\s*}{}s;
  # delete any trailing <p>
  $definition =~ s{<p>$}{}si;
  
  # store the definition
  $defs[$jndex] = $definition;
  
  # step for the next
  $jndex ++;
  
  # chop off the term and definition that was recently processed
  $TransferBuffer = substr($TransferBuffer,$endIndex + length('#'));
  # get the start of the next term
  $startIndex = index($TransferBuffer,'%');
}


use open OUT => ':utf8';

open(F,">$OutputFileName");

print F <<END;
<?xml version="1.0" encoding="utf-8"?>
<GLOSSARY>
  <INFO>
    <NAME>$title</NAME>
    <INTRO>$IntroPart</INTRO>
$infoConstants
    <ENTRIES>
END
for( my $i = 0 ; $i < $jndex ; $i++ ){
  my $concept = $term[$i];
  my $definition = $defs[$i];
  $concept    = htmlescape($concept);
  $definition = htmlescape($definition);
  print F <<END;
  <ENTRY>
  <CONCEPT>$concept</CONCEPT>
  <DEFINITION>$definition</DEFINITION>
  $entryConstants
END

if( defined $alas[$i] ){
	print F "<ALIASES>\n";
	for my $alias (  split(",",$alas[$i])  ){
		print F "<ALIAS><NAME>$alias</NAME></ALIAS>\n";
	}
	print F "</ALIASES>\n";
}

print F "</ENTRY>\n";
}
print F <<'END';
  </ENTRIES>
 </INFO>
</GLOSSARY>
END
close F;

sub htmlescape {
  my $text = shift;
  $text =~ s{<}{\&lt;}g;
  $text =~ s{>}{\&gt;}g;
  $text =~ s{"}{\&quot;}g;
  $text;
}
